syntax = "proto3";

package syft.lib.torch;

import "proto/core/plan/plan.proto";
import "proto/lib/python/collections/ordered_dict.proto";

message Module {
    string module_type = 1;
    string module_name = 2;
    string module_repr = 3;
    repeated Module children = 4;
    syft.lib.python.collections.OrderedDict state_dict = 5;
}