syntax = "proto3";
package syft.core.io;

import "proto/core/io/location.proto";
import "proto/core/io/connection.proto";

message SoloRoute {
  syft.core.io.SpecificLocation destination = 1;
  syft.core.io.VirtualClientConnection connection = 2;
}