# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/lib/petlib/ecpt.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.lib.petlib import (
    ecpt_group_pb2 as proto_dot_lib_dot_petlib_dot_ecpt__group__pb2,
)

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/lib/petlib/ecpt.proto",
    package="syft.lib.petlib",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x1bproto/lib/petlib/ecpt.proto\x12\x0fsyft.lib.petlib\x1a!proto/lib/petlib/ecpt_group.proto"?\n\x04\x45\x63Pt\x12)\n\x05group\x18\x01 \x01(\x0b\x32\x1a.syft.lib.petlib.EcPtGroup\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x62\x06proto3',
    dependencies=[
        proto_dot_lib_dot_petlib_dot_ecpt__group__pb2.DESCRIPTOR,
    ],
)


_ECPT = _descriptor.Descriptor(
    name="EcPt",
    full_name="syft.lib.petlib.EcPt",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="group",
            full_name="syft.lib.petlib.EcPt.group",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="data",
            full_name="syft.lib.petlib.EcPt.data",
            index=1,
            number=2,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=83,
    serialized_end=146,
)

_ECPT.fields_by_name[
    "group"
].message_type = proto_dot_lib_dot_petlib_dot_ecpt__group__pb2._ECPTGROUP
DESCRIPTOR.message_types_by_name["EcPt"] = _ECPT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EcPt = _reflection.GeneratedProtocolMessageType(
    "EcPt",
    (_message.Message,),
    {
        "DESCRIPTOR": _ECPT,
        "__module__": "proto.lib.petlib.ecpt_pb2"
        # @@protoc_insertion_point(class_scope:syft.lib.petlib.EcPt)
    },
)
_sym_db.RegisterMessage(EcPt)


# @@protoc_insertion_point(module_scope)
