# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/lib/torch/returntypes.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.lib.torch import tensor_pb2 as proto_dot_lib_dot_torch_dot_tensor__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/lib/torch/returntypes.proto",
    package="syft.lib.torch",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n!proto/lib/torch/returntypes.proto\x12\x0esyft.lib.torch\x1a\x1cproto/lib/torch/tensor.proto"K\n\x0bReturnTypes\x12\x10\n\x08obj_type\x18\x01 \x01(\t\x12*\n\x06values\x18\x02 \x03(\x0b\x32\x1a.syft.lib.torch.TensorDatab\x06proto3',
    dependencies=[
        proto_dot_lib_dot_torch_dot_tensor__pb2.DESCRIPTOR,
    ],
)


_RETURNTYPES = _descriptor.Descriptor(
    name="ReturnTypes",
    full_name="syft.lib.torch.ReturnTypes",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="obj_type",
            full_name="syft.lib.torch.ReturnTypes.obj_type",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="values",
            full_name="syft.lib.torch.ReturnTypes.values",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=83,
    serialized_end=158,
)

_RETURNTYPES.fields_by_name[
    "values"
].message_type = proto_dot_lib_dot_torch_dot_tensor__pb2._TENSORDATA
DESCRIPTOR.message_types_by_name["ReturnTypes"] = _RETURNTYPES
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ReturnTypes = _reflection.GeneratedProtocolMessageType(
    "ReturnTypes",
    (_message.Message,),
    {
        "DESCRIPTOR": _RETURNTYPES,
        "__module__": "proto.lib.torch.returntypes_pb2"
        # @@protoc_insertion_point(class_scope:syft.lib.torch.ReturnTypes)
    },
)
_sym_db.RegisterMessage(ReturnTypes)


# @@protoc_insertion_point(module_scope)
