syntax = "proto3";
package syft.core.common;

message CommonObject {
  syft.core.common.UID uid_object = 1;
  syft.core.common.ObjectWithID object_with_id_object = 2;
}

message ObjectWithID { syft.core.common.UID id = 1; }

message UID { bytes value = 1; }
