syntax = "proto3";

package syft.lib.pandas;

import "proto/core/common/common_object.proto";
import "proto/lib/python/dict.proto";

message PandasSeries {
  syft.core.common.UID id = 1;
  syft.lib.python.Dict series = 2;
}
