syntax = "proto3";

package syft.lib.sympc;

import "proto/lib/torch/tensor.proto";
import "proto/lib/sympc/session.proto";

message ShareTensor {
  syft.lib.torch.TensorProto tensor = 1;
  syft.lib.sympc.MPCSession session = 2;
}
