# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/lib/petlib/ecpt_group.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/lib/petlib/ecpt_group.proto",
    package="syft.lib.petlib",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n!proto/lib/petlib/ecpt_group.proto\x12\x0fsyft.lib.petlib"\x1e\n\tEcPtGroup\x12\x11\n\tgroup_nid\x18\x01 \x01(\x04\x62\x06proto3',
)


_ECPTGROUP = _descriptor.Descriptor(
    name="EcPtGroup",
    full_name="syft.lib.petlib.EcPtGroup",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="group_nid",
            full_name="syft.lib.petlib.EcPtGroup.group_nid",
            index=0,
            number=1,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=54,
    serialized_end=84,
)

DESCRIPTOR.message_types_by_name["EcPtGroup"] = _ECPTGROUP
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EcPtGroup = _reflection.GeneratedProtocolMessageType(
    "EcPtGroup",
    (_message.Message,),
    {
        "DESCRIPTOR": _ECPTGROUP,
        "__module__": "proto.lib.petlib.ecpt_group_pb2"
        # @@protoc_insertion_point(class_scope:syft.lib.petlib.EcPtGroup)
    },
)
_sym_db.RegisterMessage(EcPtGroup)


# @@protoc_insertion_point(module_scope)
